"use strict";
require('reflect-metadata');
var descriptions_1 = require("./descriptions");
var namings = require('./namings');
var Parser = (function () {
    function Parser() {
        this.serviceDescription = new descriptions_1.ServiceDescription();
    }
    Parser.prototype.parseBasePath = function (service) {
        var path = Reflect.getMetadata(namings.path, service.constructor);
        return path ? path : null;
    };
    Parser.prototype.parseMethodDescriptions = function (service) {
        var methods = [];
        var _loop_1 = function(name_1) {
            var method = service[name_1];
            if (method === service.constructor) {
                return "continue";
            }
            httpMethod = null;
            path = null;
            pathParams = [];
            headerParams = [];
            queryParams = [];
            contextParams = [];
            var methodKeys = Reflect.getMetadataKeys(method);
            methodKeys.forEach(function (k) {
                // determine the http method
                if (k === namings.httpMethodMarker) {
                    httpMethod = Reflect.getMetadata(k, method);
                }
                // determine the path
                if (k === namings.path) {
                    var rawPath = Reflect.getMetadata(k, method);
                    path = rawPath ? rawPath : null;
                }
            });
            // evaluate PathParams
            pathParams = Reflect.getMetadata(namings.pathParam, service, name_1) || [];
            // evaluate HeaderParams
            headerParams = Reflect.getMetadata(namings.headerParam, service, name_1) || [];
            // evaluate QueryParams
            queryParams = Reflect.getMetadata(namings.queryParam, service, name_1) || [];
            // evaluate ContextParams
            contextParams = Reflect.getMetadata(namings.contextParam, service, name_1) || [];
            if (httpMethod !== null) {
                md = new descriptions_1.MethodDescription(name_1, httpMethod);
                md.path = path;
                md.pathParams = pathParams;
                md.headerParams = headerParams;
                md.queryParams = queryParams;
                md.contextParams = contextParams;
                methods.push(md);
            }
        };
        var httpMethod, path, pathParams, headerParams, queryParams, contextParams, md;
        for (var _i = 0, _a = Object.getOwnPropertyNames(Object.getPrototypeOf(service)); _i < _a.length; _i++) {
            var name_1 = _a[_i];
            var state_1 = _loop_1(name_1);
            if (state_1 === "continue") continue;
        }
        return methods;
    };
    Parser.prototype.traverse = function (service) {
        var _this = this;
        var basePath = this.parseBasePath(service);
        // may be null or /
        this.serviceDescription.basePath = basePath;
        // only descriptions marked with get, post, put, delete
        var methods = this.parseMethodDescriptions(service);
        methods.forEach(function (aMethod) {
            // may have a path,
            // may have parameter - these parameters must be present oin the path
            _this.serviceDescription.addMethod(aMethod);
        });
        return this.serviceDescription;
    };
    return Parser;
}());
var ServiceParser = (function () {
    function ServiceParser() {
    }
    ServiceParser.parse = function (service) {
        var parser = new Parser();
        return parser.traverse(service);
    };
    return ServiceParser;
}());
exports.ServiceParser = ServiceParser;
//# sourceMappingURL=decorator-parser.js.map